package com.argotec.scraper;

import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.firefox.FirefoxBinary;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.firefox.FirefoxOptions;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

public class Scraper implements AutoCloseable {

    private final WebDriver webDriver;

    public static Scraper source(String url) {
        Scraper scraper = new Scraper();
        scraper.get(url);
        return scraper;
    }

    public Scraper() {
        FirefoxBinary firefoxBinary = new FirefoxBinary();
        firefoxBinary.addCommandLineOptions("--headless");

        FirefoxOptions firefoxOptions = new FirefoxOptions();
        firefoxOptions.setBinary(firefoxBinary);

        webDriver = new FirefoxDriver(firefoxOptions);
    }

    public WebDriver getWebDriver() {
        return webDriver;
    }

    public Scraper doAction(ScraperAction scraperAction) {
        scraperAction.doAction(this);
        return this;
    }

    public Scraper get(String url) {
        webDriver.get(url);
        return this;
    }

    public Scraper waitSeconds(int seconds) {
        synchronized (webDriver) {
            try {
                webDriver.wait(seconds * 1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        return this;
    }

    public Scraper switchTo() {
        String subWindowHandler = null;
        Set<String> handles = webDriver.getWindowHandles();
        for (String handle : handles) {
            subWindowHandler = handle;
        }
        webDriver.switchTo().window(subWindowHandler);
        waitSeconds(2);
        return this;
    }

    public List<ScraperElement> findRootElements(By by) {
        return webDriver.findElements(by).stream()
                .map(webElement -> new ScraperElement(this, webElement))
                .collect(Collectors.toList());
    }

    public ScraperElement findRootElement(By by) throws NoSuchElementException {
        return new ScraperElement(this, webDriver.findElement(by));
    }

    public String getCurrentUrl() {
        return webDriver.getCurrentUrl();
    }

    public String getTitle() {
        return webDriver.getTitle();
    }

    @Override
    public void close() {
        // https://bugzilla.mozilla.org/show_bug.cgi?id=1434872
        webDriver.quit();
    }

    public void quit() {
        webDriver.quit();
    }
}
