package com.argotec.scraper;

import org.openqa.selenium.*;
import org.openqa.selenium.support.ui.ISelect;
import org.openqa.selenium.support.ui.Select;
import org.openqa.selenium.support.ui.UnexpectedTagNameException;

import java.util.List;
import java.util.stream.Collectors;

public class ScraperElement {

    private final Scraper scraper;
    private final WebElement webElement;

    ScraperElement(Scraper scraper, WebElement webElement) {
        this.scraper = scraper;
        this.webElement = webElement;
    }

    public Scraper and() {
        return scraper;
    }

    public ScraperElement click() {
        webElement.click();
        return this;
    }

    public ScraperElement submit() {
        webElement.submit();
        return this;
    }

    public ScraperElement sendKeys(CharSequence... keysToSend) {
        webElement.sendKeys(keysToSend);
        return this;
    }

    public ScraperElement clear() {
        webElement.clear();
        return this;
    }

    public List<ScraperElement> findElements(By by) {
        return webElement.findElements(by)
                .stream()
                .map(e -> new ScraperElement(scraper, e))
                .collect(Collectors.toList());
    }

    public ScraperElement findElement(By by) throws NoSuchElementException {
        return new ScraperElement(scraper, webElement.findElement(by));
    }

    public ISelect toSelectElement() throws UnexpectedTagNameException {
        return new Select(webElement);
    }

    public Scraper scraper() {
        return scraper;
    }

    public WebElement getWebElement() {
        return webElement;
    }

    public String getTagName() {
        return webElement.getTagName();
    }

    public String getAttribute(String name) {
        return webElement.getAttribute(name);
    }

    public boolean isSelected() {
        return webElement.isSelected();
    }

    public boolean isEnabled() {
        return webElement.isEnabled();
    }

    public String getText() {
        return webElement.getText();
    }

    public boolean isDisplayed() {
        return webElement.isDisplayed();
    }

    public Point getLocation() {
        return webElement.getLocation();
    }

    public Dimension getSize() {
        return webElement.getSize();
    }

    public Rectangle getRect() {
        return webElement.getRect();
    }

    public String getCssValue(String propertyName) {
        return webElement.getCssValue(propertyName);
    }

    public <X> X getScreenshotAs(OutputType<X> target) throws WebDriverException {
        return webElement.getScreenshotAs(target);
    }
}
