package com.argotec.scraper;

import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import javax.annotation.Nullable;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

public class ScraperUtil {

    public static String normalizeString(String rawString) {
        rawString = rawString
                .toLowerCase()
                .replaceAll("á", "a")
                .replaceAll("é", "e")
                .replaceAll("í", "i")
                .replaceAll("ó", "o")
                .replaceAll("ú", "u")
                .replaceAll("ñ", "n")
                .replaceAll("\\s", "");

        int indexOf = rawString.indexOf("(");
        return indexOf > -1
                ? rawString.substring(0, indexOf)
                : rawString;
    }

    @Nullable
    public static WebElement findOption(ISelect select, String textToFind) {
        for (WebElement option : select.getOptions()) {
            if (textToFind.equalsIgnoreCase(normalizeString(option.getText()))) {
                return option;
            }
        }
        return null;
    }

    public static ScraperAction selectOption(String id, String optionRaw, String onErrorMessage) {
        return scraper -> {
            ISelect select = scraper.findRootElement(By.id(id)).toSelectElement();
            WebElement option = findOption(select, normalizeString(optionRaw));

            if (option == null) {
                throw new IllegalArgumentException(onErrorMessage);
            } else {
                select.selectByVisibleText(option.getText());
                scraper.waitSeconds(2);
            }
        };
    }

    public static void setDate(Scraper scraper, String id, LocalDate localDate) throws NoSuchElementException {
        ScraperElement dateEl = scraper.findRootElement(By.id(id));
        dateEl.clear();
        scraper.waitSeconds(2);

        String dateStringify = localDate.format(DateTimeFormatter.ofPattern("dd/MM/yyyy"));
        dateEl.sendKeys(dateStringify);
        scraper.waitSeconds(2);
    }
}
