package com.argotec.siar.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;

import static javax.persistence.CascadeType.ALL;

@Entity
@Table(name = "aacc")
@NoArgsConstructor
@Getter
@Setter
@EqualsAndHashCode(exclude = {"provinces", "irrigationCommunities"})
@ToString(exclude = {"provinces", "irrigationCommunities"})
public class AutonomousCommunity {

    private static final String GENERATOR_NAME = "autonomousCommunity";

    @Id
    @GenericGenerator(
            name = GENERATOR_NAME,
            strategy = "org.hibernate.id.enhanced.SequenceStyleGenerator"
            /*parameters = {
                    @Parameter(name = "sequence_name", value = "SQ-" + GENERATOR_NAME),
                    @Parameter(name = "initial_value", value = "1"),
                    @Parameter(name = "increment_size", value = "1")
            }*/
    )
    @GeneratedValue(generator = GENERATOR_NAME)
    private Long id;

    @NotNull
    @Column(nullable = false)
    private String name;

    @JsonIgnore
    @OneToMany(mappedBy = "autonomousCommunity", cascade = ALL, orphanRemoval = true)
    private List<Province> provinces = new ArrayList<>();

    @JsonIgnore
    @OneToMany(cascade = ALL, orphanRemoval = true)
    @JoinColumn(name = "aacc_id", referencedColumnName = "id")
    private List<IrrigationCommunity> irrigationCommunities = new ArrayList<>();

    public AutonomousCommunity(String name) {
        this.name = name;
    }

    public boolean addProvince(Province province) {
        return provinces.add(province);
    }

    public boolean removeProvince(Province province) {
        return provinces.remove(province);
    }

    public boolean addIrrigationCommunity(IrrigationCommunity irrigationCommunity) {
        return irrigationCommunities.add(irrigationCommunity);
    }

    public boolean removeIrrigationCommunity(IrrigationCommunity irrigationCommunity) {
        return irrigationCommunities.remove(irrigationCommunity);
    }
}
