package com.argotec.siar.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.Set;

import static javax.persistence.CascadeType.ALL;

@Entity
@Table(name = "irrigation_community")
@NoArgsConstructor
@Getter
@Setter
@EqualsAndHashCode(exclude = "stations")
@ToString(exclude = "stations")
public class IrrigationCommunity {

    private static final String GENERATOR_NAME = "irrigationCommunity";

    @Id
    @GenericGenerator(
            name = GENERATOR_NAME,
            strategy = "org.hibernate.id.enhanced.SequenceStyleGenerator"
            /*parameters = {
                    @Parameter(name = "sequence_name", value = "SQ-" + GENERATOR_NAME),
                    @Parameter(name = "initial_value", value = "1"),
                    @Parameter(name = "increment_size", value = "1")
            }*/
    )
    @GeneratedValue(generator = GENERATOR_NAME)
    private Long id;

    @NotNull
    @Column(nullable = false)
    private String name;

    @ManyToOne
    @JoinColumn(name = "aacc_id")
    private AutonomousCommunity autonomousCommunity;

    @JsonIgnore
    @OneToMany(mappedBy = "province", cascade = ALL)
    private Set<Station> stations;

    public IrrigationCommunity(String name, AutonomousCommunity autonomousCommunity) {
        this.name = name;
        this.autonomousCommunity = autonomousCommunity;
    }

    public void addStation(Station station) {
        stations.add(station);
    }

    public void removeStation(Station station) {
        stations.remove(station);
    }
}
