package com.argotec.siar.model;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;

import static com.argotec.siar.util.FormatUtil.*;

@Entity
@Table(name = "measurement")
@Data
@NoArgsConstructor
public class Measurement {

    private static final String GENERATOR_NAME = "measurement";

    @Id
    @GenericGenerator(
            name = GENERATOR_NAME,
            strategy = "org.hibernate.id.enhanced.SequenceStyleGenerator"
            /*parameters = {
                    @Parameter(name = "sequence_name", value = "SQ-" + GENERATOR_NAME),
                    @Parameter(name = "initial_value", value = "1"),
                    @Parameter(name = "increment_size", value = "1")
            }*/
    )
    @GeneratedValue(generator = GENERATOR_NAME)
    private Long id;

    private LocalDate date;

    private float averageTemp;
    private float maxTemp;
    private LocalTime hourMaxTemp;
    private float minTemp;
    private LocalTime hourMinTemp;

    private float averageHum;
    private float maxHum;
    private LocalTime hourMaxHum;
    private float minHum;
    private LocalTime hourMinHum;

    private float speedWind;
    private float directionWind;
    private float maxSpeedWind;
    private LocalTime hourMaxSpeedWind;
    private float directionMaxSpeedWind;

    private float radiation;

    private float rainfall;
    private float rainfallEffect;

    private float eto;

    // private double tempSoil30cm;
    // private double tempSoil10cm;

    @NotNull
    @Column(name = "station_id")
    private Long stationId;

    public Measurement(Long stationId, List<String> tableRow) {
        this.stationId = stationId;
        parseTableRow(tableRow);
    }

    private void parseTableRow(List<String> tableRow) {
        date = toDate(tableRow.get(0));

        averageTemp = toFloat(tableRow.get(1));
        maxTemp = toFloat(tableRow.get(2));
        hourMaxTemp = toTime(tableRow.get(3));
        minTemp = toFloat(tableRow.get(4));
        hourMinTemp = toTime(tableRow.get(5));

        averageHum = toFloat(tableRow.get(6));
        maxHum = toFloat(tableRow.get(7));
        hourMaxHum = toTime(tableRow.get(8));
        minHum = toFloat(tableRow.get(9));
        hourMinHum = toTime(tableRow.get(10));

        speedWind = toFloat(tableRow.get(11));
        directionWind = toFloat(tableRow.get(12));
        maxSpeedWind = toFloat(tableRow.get(13));
        hourMaxSpeedWind = toTime(tableRow.get(14));
        directionMaxSpeedWind = toFloat(tableRow.get(15));

        radiation = toFloat(tableRow.get(16));

        rainfall = toFloat(tableRow.get(17));
        rainfallEffect = toFloat(tableRow.get(18));

        eto = toFloat(tableRow.get(19));

        // TODO: soltitemp
    }
}
