package com.argotec.siar.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.*;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotNull;
import java.util.HashSet;
import java.util.Set;

import static javax.persistence.CascadeType.ALL;

@Entity
@Table(name = "station")
@NoArgsConstructor
@Getter
@Setter
@EqualsAndHashCode(exclude = {"province", "measurements", "crops"})
@ToString(exclude = {"province", "measurements", "crops"})
public class Station {

    private static final String GENERATOR_NAME = "station";

    @Id
    @GenericGenerator(
            name = GENERATOR_NAME,
            strategy = "org.hibernate.id.enhanced.SequenceStyleGenerator"
            /*parameters = {
                    @Parameter(name = "sequence_name", value = "SQ-" + GENERATOR_NAME),
                    @Parameter(name = "initial_value", value = "1"),
                    @Parameter(name = "increment_size", value = "1")
            }*/
    )
    @GeneratedValue(generator = GENERATOR_NAME)
    private Long id;

    @NotNull
    @Column(nullable = false)
    private String name;

    private Boolean state;

    private String municipality;

    private String parage;

    private String basin;

    private String property;

    private Integer utmx;

    private Integer utmy;

    private Integer timeZone;

    private Integer altitude;

    private String lastMeasurementDate;

    @ManyToOne
    @JoinColumn(name = "province_id")
    private Province province;

    @ManyToOne
    @JoinColumn(name = "irrigation_community_id")
    private IrrigationCommunity irrigationCommunity;

    @JsonIgnore
    @OneToMany(cascade = ALL)
    @JoinColumn(name = "station_id", referencedColumnName = "id")
    private Set<Measurement> measurements;

    @ManyToMany(cascade = ALL)
    @JoinTable(
            name = "Station_Crop",
            joinColumns = {@JoinColumn(name = "station_id")},
            inverseJoinColumns = {@JoinColumn(name = "crop_id")}
    )
    private Set<Crop> crops = new HashSet<>();

    public Station(String name, Province province, IrrigationCommunity irrigationCommunity) {
        this.name = name;
        this.province = province;
        this.irrigationCommunity = irrigationCommunity;
    }

    public void addMeasurement(Measurement measurement) {
        measurements.add(measurement);
    }

    public void removeMeasurement(Measurement measurement) {
        measurements.remove(measurement);
    }

    public void addCrop(Crop crop) {
        crops.add(crop);
    }

    public void removeCrop(Crop crop) {
        crops.remove(crop);
    }
}
