package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperAction;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.scraper.ScraperUtil.findOption;
import static com.argotec.scraper.ScraperUtil.selectOption;

@Slf4j
public class ScraperCommunities implements ScraperDataSource<AutonomousCommunity> {

    private final static String SELECT_ID = "ContentPlaceHolder1_DropDownListCCAA";

    @Override
    public List<AutonomousCommunity> scrapeAll() {
        log.info("Fetching autonomous communities using WebScrapper");
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            return findSelect(scraper)
                    .getOptions()
                    .stream()
                    .map(this::createEntityFromOption)
                    .collect(Collectors.toList());
        }
    }

    public AutonomousCommunity scrapeOne(String name) {
        log.info("Fetching autonomous community {} using WebScrapper", name);
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            WebElement optionEl = findOption(findSelect(scraper), name);
            if (optionEl != null) {
                return createEntityFromOption(optionEl);
            }
            return null;
        }
    }

    private ISelect findSelect(Scraper scraper) {
        return scraper.findRootElement(By.id(SELECT_ID)).toSelectElement();
    }

    private AutonomousCommunity createEntityFromOption(WebElement option) {
        return new AutonomousCommunity(option.getText());
    }

    public static ScraperAction selectAuCommunityAction(String autonomousCommunityName) {
        String onErrorMessage = "ERROR: Incorrectly introduced AC";
        return selectOption(SELECT_ID, autonomousCommunityName, onErrorMessage);
    }
}
