package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperAction;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.util.ScrapingUrl;
import lombok.extern.slf4j.Slf4j;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;
import java.util.stream.Collectors;

import static com.argotec.scraper.Scraper.source;
import static com.argotec.scraper.ScraperUtil.findOption;
import static com.argotec.scraper.ScraperUtil.selectOption;
import static com.argotec.siar.scraper.ScraperCommunities.selectAuCommunityAction;

@Slf4j
public class ScraperIrrigationCommunities implements ScraperDataSource<IrrigationCommunity> {

    private final static String SELECT_ID = "DropDownListProvincia";

    private final AutonomousCommunity autonomousCommunity;

    public ScraperIrrigationCommunities(AutonomousCommunity autonomousCommunity) {
        this.autonomousCommunity = autonomousCommunity;
    }

    @Override
    public List<IrrigationCommunity> scrapeAll() {
        log.info("Fetching irrigation communities of {} using WebScrapper", autonomousCommunity.getName());
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction(autonomousCommunity.getName()))
                    .doAction(selectIrrigationCommunityRadioButton());
            return findSelect(scraper)
                    .getOptions()
                    .stream()
                    .map(this::createEntityFromOption)
                    .collect(Collectors.toList());
        }
    }

    public IrrigationCommunity scrapeOne(String name) {
        log.info("Fetching irrigation community {} of {} using WebScrapper", name, autonomousCommunity.getName());
        try (Scraper scraper = source(ScrapingUrl.getUrlDq)) {
            scraper.doAction(selectAuCommunityAction(autonomousCommunity.getName()))
                    .doAction(selectIrrigationCommunityRadioButton());

            WebElement optionEl = findOption(findSelect(scraper), name);
            if (optionEl != null) {
                return createEntityFromOption(optionEl);
            }
            return null;
        }
    }

    private ISelect findSelect(Scraper scraper) {
        return scraper.findRootElement(By.id(SELECT_ID)).toSelectElement();
    }

    private IrrigationCommunity createEntityFromOption(WebElement option) {
        return new IrrigationCommunity(option.getText(), autonomousCommunity);
    }

    public static ScraperAction selectIrrigationCommunityRadioButton() {
        return scraper -> {
            scraper.findRootElement(By.id("ContentPlaceHolder1_RadioButtonCRegantes")).click();
            scraper.waitSeconds(2);
        };
    }

    public static ScraperAction selectIrrigationCommunityAction(String irrigationCommunity) {
        String onErrorMessage = "Error: Incorrectly introduced irrigation community";
        return scraper -> {
            scraper.doAction(selectIrrigationCommunityRadioButton());
            scraper.doAction(selectOption(SELECT_ID, irrigationCommunity, onErrorMessage));
        };
    }
}
