package com.argotec.siar.service.impl;

import com.argotec.siar.model.Measurement;
import com.argotec.siar.model.Station;
import com.argotec.siar.repository.MeasurementRepository;
import com.argotec.siar.scraper.ScraperMeasurements;
import com.argotec.siar.service.MeasurementService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;

@Service
@Slf4j
public class MeasurementServiceImpl implements MeasurementService {

    private final MeasurementRepository measurementRepository;

    public MeasurementServiceImpl(MeasurementRepository measurementRepository) {
        this.measurementRepository = measurementRepository;
    }

    @Override
    public List<Measurement> getAllByStationAndDateBetween(Station station, LocalDate startDate, LocalDate endDate) {
        List<Measurement> list = measurementRepository.findAllByStationIdAndDateBetween(station.getId(), startDate, endDate);

        if (list.isEmpty()) {
            list = fetchMeasurementsUsingWebScrapping(station, startDate, endDate);
        }

        return list;
    }

    private List<Measurement> fetchMeasurementsUsingWebScrapping(Station station, LocalDate startDate, LocalDate endDate) {
        List<Measurement> list = new ScraperMeasurements(station, startDate, endDate).scrapeAll();

        list.stream()
                .filter(m -> measurementRepository.findByStationIdAndDate(station.getId(), m.getDate()) == null)
                .peek(m -> log.info("Measurements added in the database"))
                .forEach(measurementRepository::save);

        return list;
    }
}
