package com.argotec.siar.service.impl;

import com.argotec.siar.model.Crop;
import com.argotec.siar.model.Station;
import com.argotec.siar.model.Variable;
import com.argotec.siar.repository.VariableRepository;
import com.argotec.siar.scraper.ScraperVariables;
import com.argotec.siar.service.StationService;
import com.argotec.siar.service.VariableService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;
import java.util.Set;

import static com.argotec.siar.util.FormatUtil.stringsAreEqualAfterNormalization;

@Service
@Slf4j
public class VariableServiceImpl implements VariableService {

    private final VariableRepository variableRepository;
    private final StationService stationService;

    public VariableServiceImpl(VariableRepository variableRepository, StationService stationService) {
        this.variableRepository = variableRepository;
        this.stationService = stationService;
    }

    @Override
    public List<Variable> getAllByStationAndCropAndDateBetween(Station station, String cropName, LocalDate startDate, LocalDate endDate, boolean ws) {
        Set<Crop> stationCrops = stationService.getCropsByStation(station, ws);

        Crop crop = stationCrops.stream()
                .filter(c -> stringsAreEqualAfterNormalization(cropName, c.getName()))
                .findFirst()
                .orElseThrow(() -> new RuntimeException(String.format("Crop '%s' not found", cropName)));

        List<Variable> list = variableRepository.findByCropIdAndDateBetween(crop.getId(), startDate, endDate);

        if (list.isEmpty()) {
            list = fetchVariablesUsingWebScrapping(station, crop, startDate, endDate);
        }

        return list;
    }

    private List<Variable> fetchVariablesUsingWebScrapping(Station station, Crop crop, LocalDate startDate, LocalDate endDate) {
        List<Variable> list = new ScraperVariables(station, crop, startDate, endDate).scrapeAll();

        list.stream()
                .filter(v -> variableRepository.findByCropIdAndDate(crop.getId(), v.getDate()) == null)
                .peek(v -> log.info("Variable added in the database"))
                .forEach(variableRepository::save);

        return list;
    }
}
