package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.util.ScrapingUrl;
import org.junit.Test;
import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;

import static com.argotec.scraper.Scraper.source;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

public class ScraperCommunitiesTest {

    @Test
    public void shouldFindCorrectlyCommunityField() {
        try (Scraper scraper = getScraper()) {
            ISelect el = fetchCommunitySelectEl(scraper);
            assertThat(el, notNullValue());
        }
    }

    @Test
    public void shouldRetrieveOptionsFromCommunityField() {
        try (Scraper scraper = getScraper()) {
            List<WebElement> els = fetchCommunitySelectEl(scraper).getOptions();
            assertFalse(els.isEmpty());
        }
    }

    @Test
    public void shouldRetrieveOptionsFromCommunityFieldAndMappedToModelEntity() {
        ScraperDataSource<AutonomousCommunity> scraperCommunities = new ScraperCommunities();

        List<AutonomousCommunity> communities = scraperCommunities.scrapeAll();

        assertFalse(communities.isEmpty());
        assertThat(communities, hasItem(hasProperty("name", equalTo("Castilla y León"))));
    }

    @Test
    public void shouldSelectACommunity() {
        String expectedCommunity = "Castilla y León";
        Scraper scraper = getScraper();

        ScraperCommunities.selectAuCommunityAction(expectedCommunity).doAction(scraper);

        String selectedCommunity = fetchCommunitySelectEl(scraper)
                .getFirstSelectedOption()
                .getText();

        assertThat(selectedCommunity, equalTo(expectedCommunity));
    }

    @Test
    public void shouldRaiseIllegalArgumentException_whenSelectedCommunityDoesNotExist() {
        Scraper scraper = getScraper();

        try {
            ScraperCommunities.selectAuCommunityAction("SOMETHING").doAction(scraper);
            fail("Has to throw IllegalArgumentException");
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("ERROR: Incorrectly introduced AC"));
        }
    }

    private Scraper getScraper() {
        return source(ScrapingUrl.getUrlDq);
    }

    private ISelect fetchCommunitySelectEl(Scraper scraper) {
        return scraper.findRootElement(By.id("ContentPlaceHolder1_DropDownListCCAA")).toSelectElement();
    }
}