package com.argotec.siar.scraper;

import com.argotec.scraper.Scraper;
import com.argotec.scraper.ScraperDataSource;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.model.Province;
import com.argotec.siar.util.ScrapingUrl;
import org.junit.Test;
import org.openqa.selenium.By;
import org.openqa.selenium.support.ui.ISelect;

import java.util.List;

import static com.argotec.scraper.Scraper.source;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

public class ScraperProvincesTest {

    @Test
    public void shouldRetrieveOptionsFromProvinceFieldAndMappedToModelEntity() {
        ScraperDataSource<Province> scraperProvinces = new ScraperProvinces(new AutonomousCommunity("Castilla y León"));

        List<Province> provinces = scraperProvinces.scrapeAll();

        assertFalse(provinces.isEmpty());
        assertThat(provinces, hasItem(hasProperty("name", equalTo("Valladolid"))));
    }

    @Test
    public void shouldSelectAProvince() {
        String expectedProvince = "Sevilla";
        Scraper scraper = getScraper();

        ScraperProvinces.selectProvinceAction(expectedProvince).doAction(scraper);

        String selectedProvince = fetchProvinceSelectEl(scraper)
                .getFirstSelectedOption()
                .getText();

        assertThat(selectedProvince, equalTo(expectedProvince));
    }

    @Test
    public void shouldRaiseIllegalArgumentException_whenSelectedProvinceDoesNotExist() {
        Scraper scraper = getScraper();

        try {
            ScraperProvinces.selectProvinceAction("SOMETHING").doAction(scraper);
            fail("Has to throw IllegalArgumentException");
        } catch (IllegalArgumentException ex) {
            assertThat(ex.getMessage(), equalTo("Error: Incorrectly introduced province"));
        }
    }

    private Scraper getScraper() {
        return source(ScrapingUrl.getUrlDq);
    }

    private ISelect fetchProvinceSelectEl(Scraper scraper) {
        return scraper.findRootElement(By.id("DropDownListProvincia")).toSelectElement();
    }
}