package com.argotec.restapi.controller;

import com.argotec.siar.model.IrrigationCommunity;
import com.argotec.siar.model.Station;
import com.argotec.siar.service.IrrigationCommunityService;
import com.argotec.siar.service.StationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/v1/irrigation-communities")
@Api(tags = {"Irrigation Community"})
public class IrrigationCommunityController {

    private final IrrigationCommunityService irrigationCommunityService;
    private final StationService stationService;

    public IrrigationCommunityController(IrrigationCommunityService irrigationCommunityService, StationService stationService) {
        this.irrigationCommunityService = irrigationCommunityService;
        this.stationService = stationService;
    }

    @GetMapping
    @ApiOperation("Get all irrigation communities")
    public List<IrrigationCommunity> getAllIrrigationCommunities(@RequestParam(required = false) String communityName,
                                                                 @RequestParam(required = false) boolean ws) {
        return communityName == null
                ? irrigationCommunityService.getAllIrrCommunities()
                : irrigationCommunityService.getAllIrrCommunitiesByAuComm(communityName, ws);
    }

    @GetMapping("/{name}")
    @ApiOperation("Get a irrigation community by name")
    public IrrigationCommunity getIrrigationCommunityByName(@PathVariable String name,
                                                            @RequestParam(required = false) String communityName,
                                                            @RequestParam(required = false) boolean ws) {
        return irrigationCommunityService.getByName(name, communityName, ws);
    }

    @GetMapping("/{name}/stations")
    @ApiOperation("Get all stations that belongs to a specific irrigation community")
    public List<Station> getStationsOfAnIrrigationCommunity(@PathVariable String name,
                                                            @RequestParam(required = false) String communityName,
                                                            @RequestParam(required = false) boolean ws) {
        IrrigationCommunity irrigationCommunity = irrigationCommunityService.getByName(name, communityName, false);
        return stationService.getAllStations(irrigationCommunity, ws);
    }
}
