package com.argotec.restapi.IT;

import com.argotec.restapi.RestapiApplication;
import com.argotec.restapi.util.TestUtil;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.repository.AutonomousCommunityRepository;
import org.hamcrest.Matchers;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;


@RunWith(SpringRunner.class)
@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT,
        classes = RestapiApplication.class)
@AutoConfigureMockMvc
//@TestPropertySource(locations = "classpath:application-integrationtest.properties")
public class AutonomousCommunityControllerIT {

//    private final static String input = "Welcome to your API REST to get meteorological data";  // final static -> create this variable only once

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private AutonomousCommunityRepository repository;

    @Test
    public void getAllCommunitiesShouldReturnFoundCommunities() throws Exception {

        AutonomousCommunity firstCommunity = new AutonomousCommunity("AACC 0");
        firstCommunity.setId(1L);
        firstCommunity.setName("AACC 1");
        repository.save(firstCommunity);

        AutonomousCommunity secondCommunity = new AutonomousCommunity("AACC 2");
        secondCommunity.setId(2L);
        repository.save(secondCommunity);

        AutonomousCommunity thirdCommunity = new AutonomousCommunity("AACC 3");
        thirdCommunity.setId(3L);
        repository.save(thirdCommunity);

        mockMvc.perform(MockMvcRequestBuilders.get("/communities"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().contentType(TestUtil.APPLICATION_JSON_UTF8))
                .andExpect(MockMvcResultMatchers.jsonPath("$", Matchers.hasSize(3)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[0].id", Matchers.is(1)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[0].name", Matchers.is("AACC 1")))
                .andExpect(MockMvcResultMatchers.jsonPath("$[1].id", Matchers.is(2)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[1].name", Matchers.is("AACC 2")))
                .andExpect(MockMvcResultMatchers.jsonPath("$[2].id", Matchers.is(3)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[2].name", Matchers.is("AACC 3")));
    }
}


