package com.argotec.restapi.controller;

import com.argotec.restapi.RestapiApplication;
import com.argotec.restapi.util.TestUtil;
import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.service.AutonomousCommunityService;
import org.hamcrest.Matchers;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.BDDMockito;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.test.web.servlet.result.MockMvcResultMatchers;

import java.util.ArrayList;
import java.util.List;

import static org.mockito.ArgumentMatchers.any;


@RunWith(SpringRunner.class)
@SpringBootTest(classes = RestapiApplication.class)
@AutoConfigureMockMvc
public class AutonomousCommunityControllerTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private AutonomousCommunityService service;

    @Test
    public void findAllCommunitiesShouldReturnFoundCommunities() throws Exception {

        List<AutonomousCommunity> allCommunities = new ArrayList<>();

        AutonomousCommunity firstCommunity = new AutonomousCommunity("AACC 1");
        firstCommunity.setId(1L);
        allCommunities.add(firstCommunity);
        AutonomousCommunity secondCommunity = new AutonomousCommunity("AACC 2");
        secondCommunity.setId(2L);
        allCommunities.add(secondCommunity);
        AutonomousCommunity thirdCommunity = new AutonomousCommunity("AACC 3");
        thirdCommunity.setId(3L);
        allCommunities.add(thirdCommunity);

        BDDMockito.given(service.getAllCommunities(any())).willReturn(allCommunities);

        mockMvc.perform(MockMvcRequestBuilders.get("/communities"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().contentType(TestUtil.APPLICATION_JSON_UTF8))
                .andExpect(MockMvcResultMatchers.jsonPath("$", Matchers.hasSize(3)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[0].id", Matchers.is(1)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[0].name", Matchers.is("AACC 1")))
                .andExpect(MockMvcResultMatchers.jsonPath("$[1].id", Matchers.is(2)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[1].name", Matchers.is("AACC 2")))
                .andExpect(MockMvcResultMatchers.jsonPath("$[2].id", Matchers.is(3)))
                .andExpect(MockMvcResultMatchers.jsonPath("$[2].name", Matchers.is("AACC 3")));
    }

    @Test
    public void findByNameShouldReturnFoundCommunity() throws Exception {
        String name = "AACC Name";
        AutonomousCommunity communityFound = new AutonomousCommunity("AACC Name");
        communityFound.setId(7L);

        Mockito.when(service.getByName(name)).thenReturn(communityFound);

        mockMvc.perform(MockMvcRequestBuilders.get("/communities/{name}", "AACC Name"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().contentType(TestUtil.APPLICATION_JSON_UTF8))
                .andExpect(MockMvcResultMatchers.jsonPath("$.id", Matchers.is(7)))
                .andExpect(MockMvcResultMatchers.jsonPath("$.name", Matchers.is("AACC Name")));
    }


    @Test
    public void findAllCommunitiesToStringShouldReturnStringFoundCommunities() throws Exception {

        String contentJson = "[\"AutonomousCommunity[id=1, name='AACC S1']\\n\",\"AutonomousCommunity[id=2, name='AACC S2']\\n\"]";

        List<AutonomousCommunity> sAllCommunities = new ArrayList<>();

        AutonomousCommunity sFirstCommunity = new AutonomousCommunity("AACC S1");
        sFirstCommunity.setId(1L);
        sAllCommunities.add(sFirstCommunity);

        AutonomousCommunity sSecondCommunity = new AutonomousCommunity("AACC S2");
        sSecondCommunity.setId(2L);
        sAllCommunities.add(sSecondCommunity);

        Mockito.when(service.getAllCommunities(any())).thenReturn(sAllCommunities);

        mockMvc.perform(MockMvcRequestBuilders.get("/communities/tostring"))
                .andExpect(MockMvcResultMatchers.status().isOk())
                .andExpect(MockMvcResultMatchers.content().contentType(TestUtil.APPLICATION_JSON_UTF8))
                .andExpect(MockMvcResultMatchers.jsonPath("$", Matchers.hasSize(2)))
                .andExpect(MockMvcResultMatchers.content().string(contentJson));

    }
}