package com.argotec.siar.service;

import com.argotec.siar.model.AutonomousCommunity;
import com.argotec.siar.repository.AutonomousCommunityRepository;
import com.argotec.siar.service.impl.AutonomousCommunityServiceImpl;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.ArrayList;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;


public class AutonomousCommunityServiceImplWithMockitoTest {

    private final List<AutonomousCommunity> allCommunities = new ArrayList<>();
    private AutonomousCommunity fCommunity;

    @Before
    public void setUp() {
        fCommunity = new AutonomousCommunity("AACC TSM1");
        AutonomousCommunity sCommunity = new AutonomousCommunity("AACC TSM2");
        fCommunity.setId(1L);
        sCommunity.setId(2L);
        allCommunities.add(fCommunity);
        allCommunities.add(sCommunity);
    }

    @Test
    public void findAllCommunitiesShouldReturnAllCommunitiesWithMockito() {
        AutonomousCommunityRepository repository = Mockito.mock(AutonomousCommunityRepository.class);
        Mockito.when(repository.findAll()).thenReturn(allCommunities);
        AutonomousCommunityServiceImpl service = new AutonomousCommunityServiceImpl(repository);
        List<AutonomousCommunity> currentCommunities = service.getAllCommunities(false);
        MatcherAssert.assertThat(allCommunities, Matchers.is(currentCommunities));
    }

    @Test
    public void findByNameShouldReturnThisCommunityWithMockito() {
        AutonomousCommunityRepository repository = Mockito.mock(AutonomousCommunityRepository.class);
        Mockito.when(repository.findByName("AACC TSM1")).thenReturn(fCommunity);
        AutonomousCommunityServiceImpl service = new AutonomousCommunityServiceImpl(repository);
        AutonomousCommunity communityFound = service.getByName("AACC TSM1");
        assertThat(fCommunity, is(communityFound));
    }
}

